@@ -1,19 +1,19 @@
+use v5.10;
+
 package Mac::PropertyList::ReadBinary;
 use strict;
 use warnings;
 
-use vars qw( $VERSION );
-
 use Carp;
 use Data::Dumper;
 use Encode            qw(decode);
 use Mac::PropertyList;
 use Math::BigInt;
-use MIME::Base64      qw(decode_base64);
 use POSIX             qw(SEEK_END SEEK_SET);
-use XML::Entities     ();
 
-$VERSION = '1.41_01';
+our $VERSION = '1.603_02';
+
+my $Debug = $ENV{PLIST_DEBUG} || 0;
 
 __PACKAGE__->_run( @ARGV ) unless caller;
 
@@ -50,7 +50,7 @@ that data structure and get back the plist as XML (but not binary
 yet). If you want to change the structure inbetween that's your
 business. :)
 
-See C<Mac::PropertyList> for more details.
+See L<Mac::PropertyList> for more details.
 
 =head2 Methods
 
@@ -87,8 +87,7 @@ Returns the C<Mac::PropertyList> data structure.
 
 sub plist            { $_[0]->{parsed}             }
 
-sub _object_size
-	{
+sub _object_size {
 	$_[0]->_trailer->{object_count} * $_[0]->_trailer->{offset_size}
 	}
 
@@ -127,8 +126,7 @@ sub _get_filehandle {
 	$fh;
 	}
 
-sub _read_plist_trailer
-	{
+sub _read_plist_trailer {
 	my $self = shift;
 
 	seek $self->_fh, -32, SEEK_END;
@@ -143,8 +141,7 @@ sub _read_plist_trailer
 	$self->{trailer} = \%hash;
 	}
 
-sub _get_offset_table
-	{
+sub _get_offset_table {
 	my $self = shift;
 
     seek $self->_fh, $self->_trailer->{table_offset}, SEEK_SET;
@@ -160,18 +157,14 @@ sub _get_offset_table
 
 	$self->{offsets} = \@offsets;
 
-    if( $self->_trailer->{offset_size} == 3 )
-    	{
+    if( $self->_trailer->{offset_size} == 3 ) {
 		@offsets = map { hex } @offsets;
    	 	}
-
 	}
 
 sub _read_object_at_offset {
 	my( $self, $offset ) = @_;
 
-	my @caller = caller(1);
-
     seek $self->_fh, ${ $self->_offsets }[$offset], SEEK_SET;
 
     $self->_read_object;
@@ -190,43 +183,56 @@ my %singletons = (
     # implementation in CFBinaryPList.c but Apple's actual code has no
     # support for it at all, either reading or writing, so it's
     # probably not important to implement.
-
-);
+	);
 
 my $type_readers = {
-
 	0 => sub { # the odd balls
 		my( $self, $length ) = @_;
 
 		return $singletons{ $length } if exists $singletons{ $length };
-		
+
 		croak ( sprintf "Unknown type byte %02X\n", $length );
     	},
 
 	1 => sub { # integers
-		my( $self, $length ) = @_;
-		croak "Integer > 8 bytes = $length" if $length > 3;
+		my( $self, $power_of_2 ) = @_;
 
-		my $byte_length = 1 << $length;
+		croak "Integer with <$power_of_2> bytes is not supported" if $power_of_2 > 4;
+
+		my $byte_length = 1 << $power_of_2;
 
 		my( $buffer, $value );
 		read $self->_fh, $buffer, $byte_length;
 
-		my @formats = qw( C n N NN );
-		my @values = unpack $formats[$length], $buffer;
+		my @formats = qw( C n N NN NNNN );
+		my @values = unpack $formats[$power_of_2], $buffer;
 
-		if( $length == 3 )
-			{
+		if( $power_of_2 == 3 ) { # 64 bits
 			my( $high, $low ) = @values;
 
 			my $b = Math::BigInt->new($high)->blsft(32)->bior($low);
-			if( $b->bcmp(Math::BigInt->new(2)->bpow(63)) > 0)
-				{
+			if( $b->bcmp(Math::BigInt->new(2)->bpow(63)) >= 0) {
 				$b -= Math::BigInt->new(2)->bpow(64);
 				}
 
 			@values = ( $b );
 			}
+		elsif( $power_of_2 == 4 ) { # 128 bits
+		    # 128 bits aren't part of the public API, but apparently
+		    # they are out there.
+			my( $highest, $higher, $high, $low ) = @values;
+			my $b = Math::BigInt
+				->new($highest)
+				->blsft(32)->bior($higher)
+				->blsft(32)->bior($high)
+				->blsft(32)->bior($low);
+
+			if( $b->bcmp(Math::BigInt->new(2)->bpow(127)) >= 0) {
+				$b -= Math::BigInt->new(2)->bpow(128);
+				}
+
+			@values = ( $b );
+			}
 
 		return Mac::PropertyList::integer->new( $values[0] );
 		},
@@ -298,6 +304,18 @@ my $type_readers = {
 		return Mac::PropertyList::ustring->new( $buffer );
 		},
 
+	8 => sub { # UIDs
+		my( $self, $length ) = @_;
+
+		my $byte_length = $length + 1;
+
+		read $self->_fh, ( my $buffer ), $byte_length;
+
+		my $value = unpack 'H*', $buffer;
+
+		return Mac::PropertyList::uid->new( $value );
+		},
+
 	a => sub { # array
 		my( $self, $elements ) = @_;
 
@@ -346,16 +364,14 @@ my $type_readers = {
 		},
 	};
 
-sub _read_object
-	{
+sub _read_object {
 	my $self = shift;
-
     my $buffer;
+
     croak "read() failed while trying to get type byte! $!"
     	unless read( $self->_fh, $buffer, 1) == 1;
 
     my $length = unpack( "C*", $buffer ) & 0x0F;
-
     $buffer    = unpack "H*", $buffer;
     my $type   = substr $buffer, 0, 1;
 
@@ -382,20 +398,22 @@ Some of the ideas are cribbed from CFBinaryPList.c
 
 This project is in Github:
 
-	git://github.com/briandfoy/mac-propertylist.git
+	https://github.com/briandfoy/mac-propertylist.git
 
 =head1 CREDITS
 
 =head1 AUTHOR
 
-brian d foy, C<< <bdfoy@cpan.org> >>
+brian d foy, C<< <briandfoy@pobox.com> >>
+
+Tom Wyant added support for UID types.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright © 2004-2015, brian d foy <bdfoy@cpan.org>. All rights reserved.
+Copyright © 2004-2025, brian d foy <briandfoy@pobox.com>. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+it under the terms of the Artistic License 2.0.
 
 =cut
 
